/*******************************************************************************
 * Copyright 2015 - 2023 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europäischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie dürfen dieses Werk ausschließlich gemäß dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschränkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrcodehandler;

/**
 * Interface für alle QrCodeHandlers
 *
 * @author Frank Münster
 */
public interface QrCodeHandler {

    /**
     * Wird aufgerufen, um zu überprüfen, ob der Handler den QR-Code verarbeiten
     * kann.
     *
     * @param qrCode
     *            - Zu bearbeitender QR-Code
     * @return <b>true</b>, falls der Handler den QR-Code bearbeiten kann,
     *         <b>false</b> sonst
     */
    boolean canHandle(String qrCode);

    /**
     * Liefert der Namen des Handlers zurück. Der Name ist frei wählbar, sollte
     * aber aussagekräftig sein
     *
     * @return <String> - Name des Handlers
     */
    String getHandlerName();

    /**
     * Wird aufgerufen, um einen QR-Code zu bearbeiten
     *
     * @param qrCode
     *            Der QR-Code als <b>String</b>
     * @throws QrCodeHandlerException
     *             Falls der QR-Code nicht bearbeitet werden kann. Vorher
     *             mittels <b>canHandle()</b> überprüfen
     */
    void handleQrCode(String qrCode) throws QrCodeHandlerException;

    /**
     * Zeigt an, ob der Handler für Überweisungen genutzt wird. Dies wird bei
     * der Verarbeitung von PDF Dateien benötigt.
     *
     * @return <b>true</b>, falls der Handler Überweisungen bearbeitet,
     *         <b>false</b> sonst
     */
    boolean isUeberweisung();
}
