/*******************************************************************************
 * Copyright 2015 - 2023 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europäischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie dürfen dieses Werk ausschließlich gemäß dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschränkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrcodehandler;

import java.util.ArrayList;
import java.util.List;
import java.util.ListIterator;

import de.frankmuenster.jameica.qrscanner.utils.I18N;
import de.willuhn.logging.Level;
import de.willuhn.logging.Logger;

/**
 * Registry fuer die QR-Code Handler Neue Handler muessen sich mittels
 * register() in der Registry anmelden
 *
 * @author Frank Münster
 * @version 1.1
 */
public final class QrCodeHandlerRegsistry {

    /**
     * Liste der registrierten QR-Code-Handlers
     */
    private static List<QrCodeHandler> qrCodeHandlers = new ArrayList<>();

    /**
     * Liefert einen passenden QrCodeHandler zum <b>qrCode</b>, falls vorhanden.
     *
     * @param qrCode
     *            <String> mit QR-Code zu dem ein Handler gesucht wird.
     * @return Handler von Typ <i>QrCodeHandler</i>, der den String bearbeiten
     *         kann.
     */
    public static QrCodeHandler getQrCodeHandler(final String qrCode) {
        QrCodeHandler handler = null;
        final ListIterator<QrCodeHandler> iter = QrCodeHandlerRegsistry.qrCodeHandlers.listIterator();
        while (iter.hasNext()) {
            handler = iter.next();
            if (handler.canHandle(qrCode)) {
                break;
            }
        }
        //
        return handler;
    }

    /**
     * Untersucht Eingabe-String, ob er mit einem Handler fuer Ueberweisung
     * bearbeitet werden kann
     *
     * @param qrCode
     *            <String> mit QR-Code der untersucht werden soll.
     * @return <b>true</b> wenn QR-Code als Überweisung bearbeitet werden kann,
     *         <b>false</> sonst.
     */
    public static boolean isUeberweisung(final String qrCode) {
        Boolean isUeberweisung = false;
        final ListIterator<QrCodeHandler> iter = QrCodeHandlerRegsistry.qrCodeHandlers.listIterator();
        while (iter.hasNext()) {
            final QrCodeHandler handler = iter.next();
            if (handler.canHandle(qrCode) && handler.isUeberweisung()) {
                isUeberweisung = true;
                break;
            }
        }
        return isUeberweisung;
    }

    /**
     * Neue Qr-Code Handler koennen sich hier in der Registry einschreiben.
     *
     * @param handler
     *            Handler, der das Interface <i>QrCodeHandler</i> implementiert.
     */
    public static void register(final QrCodeHandler handler) {
        if (Logger.isLogging(Level.INFO)) {
            Logger.info(I18N.tr("QrCodeHandlerRegsistry.registerHandler", handler.getHandlerName())); //$NON-NLS-1$
        }
        QrCodeHandlerRegsistry.qrCodeHandlers.add(handler);
    }

    private QrCodeHandlerRegsistry() {
    }
}
