/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrcodehandler.bezahlcode;

import java.util.Locale;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.apache.commons.lang.StringUtils;
import org.apache.commons.lang.SystemUtils;

import de.frankmuenster.jameica.qrscanner.utils.I18N;

/**
 * @author Frank Mnster
 */
final class BezahlCodeCommenFunctions {

    /**
     * Text zur Nachfrage, ob Grobuchstaben umgewandelt werden sollen. {0} wird beim erstellen des Dialoges ersetzt, um
     * Nachfrage zu verhindern.
     */
    public final static String QUESTION_TEXT = I18N.tr("BezahlCodeCommenFunctions.reCapitaliseNameLine1") //$NON-NLS-1$
            + SystemUtils.LINE_SEPARATOR + I18N.tr("BezahlCodeCommenFunctions.reCapitaliseNameLine2") //$NON-NLS-1$
            + SystemUtils.LINE_SEPARATOR + SystemUtils.LINE_SEPARATOR
            + I18N.tr("BezahlCodeCommenFunctions.reCapitaliseNameLine3") //$NON-NLS-1$
            + SystemUtils.LINE_SEPARATOR;

    /**
     * Standardkonstruktor
     */
    private BezahlCodeCommenFunctions() {
        // Absichtlich leer
    }

    /**
     * Speichert die Query-Elemente einer URL in einem Map Object
     *
     * @param query
     *            URL mit query Elementen
     * @return Map-Object mit query-Elementen als named pairs.
     */
    public static Map<String, String> getQueryMap(final String query) {
        final String[] params = query.split("&"); //$NON-NLS-1$
        final Map<String, String> map = new ConcurrentHashMap<>();
        for (final String param : params) {
            final String[] values = param.split("="); //$NON-NLS-1$
            if (values.length > 0) {
                final String name = values[0].toLowerCase(Locale.getDefault());
                String value = StringUtils.EMPTY;
                if (values.length > 1) { // NOPMD
                    value = param.split("=")[1]; //$NON-NLS-1$
                }
                map.put(name, value);
            }
        }
        return map;
    }

    /**
     * Wandelt den Eingabe-String in erstes Wortzeichen Gro- und den Rest in Kleinbuchstaben um.
     *
     * @param text
     * @return String mit jeweils ersten Wortzeichen in Grobuchstabe
     */
    public static String formatString(final String text) {
        String formatedText;
        if (StringUtils.isEmpty(text)) {
            formatedText = text;
        } else {
            final StringBuilder cleanName = new StringBuilder();
            final String[] nameTildes = text.split(" "); //$NON-NLS-1$
            for (int i = 0; i < nameTildes.length; i++) {
                if (nameTildes[i].length() > 3) {  // NOPMD
                    cleanName.append(nameTildes[i].substring(0, 1).toUpperCase(Locale.getDefault()));
                    cleanName.append(nameTildes[i].substring(1).toLowerCase(Locale.getDefault())).append(' ');
                } else {
                    cleanName.append(nameTildes[i]).append(' ');
                }
            }
            formatedText = cleanName.substring(0, cleanName.length() - 1);
        }
        return formatedText;
    }
}
