/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/

package de.frankmuenster.jameica.qrscanner.gui.navigation;

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.swt.graphics.Image;

import de.frankmuenster.jameica.qrscanner.utils.I18N;
import de.willuhn.datasource.GenericIterator;
import de.willuhn.datasource.GenericObject;
import de.willuhn.datasource.GenericObjectNode;
import de.willuhn.datasource.pseudo.PseudoIterator;
import de.willuhn.jameica.gui.Action;
import de.willuhn.jameica.gui.Item;
import de.willuhn.jameica.gui.NavigationItem;
import de.willuhn.jameica.gui.util.SWTUtil;

/**
 * QRCodeScanner Navigation Items
 *
 * @author Frank Mnster
 */
public class QrCodeScannerItem implements NavigationItem {

    /**
     * Das Elternnavigation Item unter das wir den QR-Codescanner hngen
     */
    private final NavigationItem parent;

    /**
     * Die auszufhrende Aktion
     */
    private final Action action;

    /**
     * Der MenText
     */
    private final transient String navitext;

    /**
     * Liste mit Untermeneintrgen
     */
    private final List<Item> children;

    /**
     * Das Menicon
     */
    private final transient String icon;

    /**
     * Menpunkt aktivert
     */
    private Boolean enabled;

    /**
     * Erstellt ein QrCodeScanner Navigation Item.<br/>
     * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i>
     * initialisiert.
     *
     * @param item
     *            ist das Elternelement des Navigationsitems
     * @param navitext
     *            der Text, der angezeigt wird
     * @param action
     *            die KLasse, die das das Interface <b>Action</b> implemtiert
     *            und durch das NavigationItem ausgefhrt werden soll.
     */
    public QrCodeScannerItem(final NavigationItem item, final String navitext, final Action action) {
	this(item, navitext, action, null);
    }

    /**
     * Erstellt ein QrCodeScanner Navigation Item.<br/>
     * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i>
     * initialisiert.
     *
     * @param item
     *            ist das Elternelement des Navigationsitems
     * @param navitext
     *            der Text, der angezeigt wird
     * @param action
     *            die KLasse, die das das Interface <b>Action</b> implemtiert
     *            und durch das NavigationItem ausgefhrt werden soll.
     * @param icon
     *            String des Icon-Bildes des NavigationItems
     */
    public QrCodeScannerItem(final NavigationItem item, final String navitext, final Action action, final String icon) {
	this(item, navitext, action, icon, true);
    }

    /**
     * Erstellt ein QrCodeScanner Navigation Item.<br/>
     * <b>Action</b> wird mit <i>null</i> und <b>enabled</b> mit <i>true</i>
     * initialisiert.
     *
     * @param item
     *            ist das Elternelement des Navigationsitems
     * @param navitext
     *            der Text, der angezeigt wird
     * @param action
     *            die KLasse, die das das Interface <b>Action</b> implementiert
     *            und durch das NavigationItem ausgefhrt werden soll.
     * @param icon
     *            String des Icon-Bildes des NavigationItems
     * @param enabled
     *            setzt den Aktiv-Status des Items
     */
    public QrCodeScannerItem(final NavigationItem item, final String navitext, final Action action, final String icon,
	    final Boolean enabled) {
	this.parent = item;
	this.action = action;
	this.navitext = navitext;
	this.icon = icon;
	this.enabled = enabled;
	this.children = new ArrayList<>();
    }

    /**
     * @see de.willuhn.jameica.gui.Item#addChild(de.willuhn.jameica.gui.Item)
     */
    @Override
    public void addChild(final Item item) throws RemoteException {
	throw new RemoteException(I18N.tr("QrCodeScannerItem.noChildAllowed")); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.GenericObject#equals(de.willuhn.datasource.
     *      GenericObject)
     */
    @Override
    public boolean equals(final GenericObject arg0) throws RemoteException { // NOPMD
	boolean equal = false;
	if (arg0 instanceof QrCodeScannerItem) {
	    equal = this.getID().equals(arg0.getID());
	}
	return equal;
    }

    /**
     * @see de.willuhn.jameica.gui.Item#getAction()
     */
    @Override
    public Action getAction() throws RemoteException {
	return this.action;
    }

    /**
     * @see de.willuhn.datasource.GenericObject#getAttribute(java.lang.String)
     */
    @Override
    public Object getAttribute(final String arg0) throws RemoteException {
	return this.getName();
    }

    /**
     * @see de.willuhn.datasource.GenericObject#getAttributeNames()
     */
    @Override
    public String[] getAttributeNames() throws RemoteException {
	return new String[] { "name" }; //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.GenericObjectNode#getChildren()
     */
    @Override
    public GenericIterator<?> getChildren() throws RemoteException {
	return PseudoIterator.fromArray(this.children.toArray(new QrCodeScannerItem[this.children.size()]));
    }

    /**
     * @see de.willuhn.jameica.gui.extension.Extendable#getExtendableID()
     */
    @Override
    public String getExtendableID() {
	return this.getID();
    }

    /**
     * @see de.willuhn.jameica.gui.NavigationItem#getIconClose()
     */
    @Override
    public Image getIconClose() throws RemoteException {
	return SWTUtil.getImage(this.icon == null ? "webcam_add.png" : this.icon); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.jameica.gui.NavigationItem#getIconOpen()
     */
    @Override
    public Image getIconOpen() throws RemoteException {
	return SWTUtil.getImage(this.icon == null ? "webcam_add.png" : this.icon); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.GenericObject#getID()
     */
    @Override
    public String getID() {
	return this.getClass().getName() + "." + this.getName(); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.jameica.gui.Item#getName()
     */
    @Override
    public String getName() {
	return this.navitext;
    }

    /**
     * @see de.willuhn.datasource.GenericObjectNode#getParent()
     */
    @Override
    public GenericObjectNode getParent() throws RemoteException {
	return this.parent;
    }

    /**
     * @see de.willuhn.datasource.GenericObjectNode#getPath()
     */
    @Override
    public GenericIterator<?> getPath() throws RemoteException {
	@SuppressWarnings("unchecked")
	final List<NavigationItem> list = PseudoIterator.asList(this.parent.getPath());
	list.add(this);
	return PseudoIterator.fromArray(list.toArray(new NavigationItem[0]));
    }

    /**
     * @see de.willuhn.datasource.GenericObjectNode#getPossibleParents()
     */
    @Override
    public GenericIterator<?> getPossibleParents() throws RemoteException {
	throw new RemoteException(I18N.tr("QrCodeScannerItem.notImplemented")); //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.GenericObject#getPrimaryAttribute()
     */
    @Override
    public String getPrimaryAttribute() throws RemoteException {
	return "name"; //$NON-NLS-1$
    }

    /**
     * @see de.willuhn.datasource.GenericObjectNode#hasChild(de.willuhn.datasource
     *      .GenericObjectNode)
     */
    @Override
    public boolean hasChild(final GenericObjectNode arg0) throws RemoteException {
	return false;
    }

    /**
     * @see de.willuhn.jameica.gui.Item#isEnabled()
     */
    @Override
    public boolean isEnabled() throws RemoteException {
	return this.enabled;
    }

    /**
     * @see de.willuhn.jameica.gui.NavigationItem#isExpanded()
     */
    @Override
    public boolean isExpanded() throws RemoteException {
	return false;
    }

    /**
     * @see de.willuhn.jameica.gui.Item#setEnabled(boolean, boolean)
     */
    @Override
    public void setEnabled(final boolean enabled, final boolean recursive) throws RemoteException {
	this.enabled = enabled;
    }
}
