/*******************************************************************************
 * Copyright 2015 - 2018 fmCreation
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald diese von der
 * Europischen Kommission genehmigt wurden - Folgeversionen der EUPL
 * ("Lizenz"); Sie drfen dieses Werk ausschlielich gem dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 *   https://progs.frankmuenster.de/eupl_v1-2_de
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften gefordert oder in
 * schriftlicher Form vereinbart, wird die unter der Lizenz verbreitete Software
 * "so wie sie ist", OHNE JEGLICHE GEWHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrcklich oder stillschweigend - verbreitet. Die sprachspezifischen
 * Genehmigungen und Beschrnkungen unter der Lizenz sind dem Lizenztext zu
 * entnehmen.
 ******************************************************************************/
package de.frankmuenster.jameica.qrscanner.ipcam;

import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;

import org.apache.commons.lang.StringUtils;

import com.github.sarxos.webcam.ds.ipcam.IpCamDevice;
import com.github.sarxos.webcam.ds.ipcam.IpCamMode;

import de.willuhn.logging.Logger;

/**
 * Spezielle IpCamDevice Implementierung f&uuml;r Iphaone IP Camera App. &Uuml;berschreibt isOnline.
 *
 * @author Frank Mnster
 */
public class IphoneIpCameraCamDevice extends IpCamDevice {

    /**
     * Kameraname
     */
    public static final String CAM_NAME = "Iphone IP Camera"; //$NON-NLS-1$

    /**
     * HTTP-timeout f&uuml;r Ping Anfrage
     */
    private final static int TIMEOUT_PING = 500;

    /**
     * Der Host des Streams. Kann auch IP-Adresse sein.
     */
    private transient String host;

    /**
     * Der Port des Streams.
     */
    private transient int port;

    /**
     * Erstellt ein DroidCam Instance durch Angabe der IP und des Ports.
     *
     * @param host
     *            Der Host des Streams. Kann auch IP-Adresse sein.
     * @param port
     *            Der Port des Streams.
     * @throws MalformedURLException
     */
    public IphoneIpCameraCamDevice(final String host, final int port) throws MalformedURLException {
        this(IphoneIpCameraCamDevice.getCamUrl(host, port));
    }

    /**
     * Erstellt ein DroidCam Instance durch Angabe der URL.
     *
     * @param url
     *            Die URL zu dem mjpegfeed Stream der DroidCam App. Das Suffix mjpegfeed muss mit angegeben werden.
     */
    public IphoneIpCameraCamDevice(final URL url) {
        super(IphoneIpCameraCamDevice.CAM_NAME, url, IpCamMode.PUSH);
        this.host = url.getHost();
        this.port = url.getPort();
    }

    /**
     * Erstellt ein URL aus angegeben Host und Port.
     *
     * @param host
     *            Der Host des Streams. Kann auch IP-Adresse sein.
     * @param port
     *            Der Port des Streams.
     * @return Ein URL Objekt, mit Host und Port, das auf den DriodCam mjpegfeed Stream zeigt.
     * @throws MalformedURLException
     */
    private static URL getCamUrl(final String host, final int port) throws MalformedURLException {
        final URL url = new URL("http", host, port, StringUtils.EMPTY); //$NON-NLS-1$
        Logger.debug("IPCamera URL: " + url.toString()); //$NON-NLS-1$
        return url;
    }

    /**
     * Pings the IPCamera URL. This effectively sends a GET request and returns <code>true</code> if the response code
     * is in the 200-399 range.
     *
     * @return <code>true</code> if the given HTTP URL has returned response code 200-399 on a HEAD request within the
     *         given TIMEOUT_PING, otherwise <code>false</code>.
     */
    @Override
    public boolean isOnline() {
        boolean isOnline;
        try {
            final URL url = new URL("http", this.host, this.port, StringUtils.EMPTY); //$NON-NLS-1$
            final HttpURLConnection connection = (HttpURLConnection) url.openConnection();
            connection.setConnectTimeout(IphoneIpCameraCamDevice.TIMEOUT_PING);
            connection.setReadTimeout(IphoneIpCameraCamDevice.TIMEOUT_PING);
            // connection.setRequestMethod("HEAD");
            final int responseCode = connection.getResponseCode();
            connection.disconnect();
            isOnline = 200 <= responseCode && responseCode <= 399;
        } catch (final IOException exception) {
            isOnline = false;
        }
        return isOnline;
    }
}
